==> fferrorhandler-2.0.3.tgz: package/middleware.js <==
const handlers = require("./handlers");
const errorTypes = require("./errorTypes");
const { slackChannels } = require("./constants");

const appName = process.env.PAPERTRAIL_PROGRAM || "UNKNOWN";

/**
 * Checks that a slack channel exist in {@link slackChannels}
 * @param {string} - The slack channel to check against {@link slackChannels}
 * @returns {boolean} - True for valid, false for invalid
 */
function _isSlackChannelValid(slackChannel) {
    return Object.values(slackChannels).includes(slackChannel);
}

/** 
 * Gets the slack channel to send the error to for {@link errorTypes.NonOperationalError}. 
 *
 * If the slack channel is set in the errorTypes.NonOperationalError constructor (error.slackChannel), it will use that. 
 * If it is not set on the error, it will check to see if process.env.NON_OPERATIONAL_SLACK_CHANNEL is set, and will use that if present.
 * If neither of those is set, it will default to alerts_warnings. 
 *
 * It also validates that the slack channel passed in is a valid slack channel that exist in {@link slackChannels}
 * @param {errorTypes.NonOperationalError} error - The NonOperationalError to get the slack channel for 
 * @returns {string} - The slack channel to send the error to. 
 * @private
 * */
function _getNonOperationalSlackChannel(error) {
    const errorSlackChannel = error.slackChannel;
    const isSlackChannelSetOnError = (
        errorSlackChannel !== undefined &&
        errorSlackChannel !== null
    );

    if (isSlackChannelSetOnError) {
        const isSlackChannelValid = module.exports._isSlackChannelValid(errorSlackChannel);
        if (!isSlackChannelValid) {
            console.error("Invalid slack channel set on error.slackChannel, defaulting to alerts_warnings");
            return slackChannels.WARNING;
        } else {
            return errorSlackChannel;
        }
    }

    const envSlackChannel = process.env.NON_OPERATIONAL_SLACK_CHANNEL;
    const isSlackChannelSetInEnv = (
        envSlackChannel !== undefined &&
        envSlackChannel !== null
    );
    if (isSlackChannelSetInEnv) {
        const isSlackChannelValid = module.exports._isSlackChannelValid(envSlackChannel);
        if (!isSlackChannelValid) {
            console.error("Invalid slack channel set in process.env.NON_OPERATIONAL_SLACK_CHANNEL, defaulting to alerts_warnings");
            return slackChannels.WARNING;
        } else {
            return envSlackChannel;
        }
    }

    return slackChannels.WARNING;
}

/**
 * Middleware that handles logging errors to papertrail, and sending non operational errors to slack. 
 * @async
 * @param {Error|import("./errorTypes").OperationalError|import("./errorTypes").NonOperationalError|import("errorTypes").HttpError} error - The error that occurred
 * @param {import("express").Request} req  - The express request object
 * @param {import("express").Response} res  - The express response object
 * @param {import("express".NextFunction)} next - The express next function
 * @returns {Promise<void>}
 * */
async function expressLogErrors(error, req, res, next) {
    /** 
     * Custom error types in {@see errorTypes} combine the stack with optional additional data and put it on error.data 
     * if we have error.data, use that as the error details, if we don't, use the error stack by itself.
     * @type {string}
     * */
    const additionalData = error.data || error.stack;

    if (!error.isOperational) {
        const slackChannel = _getNonOperationalSlackChannel(error);
        /** If this is an unexpected error, set the slack channel and send to slack and papertrail */
        handlers.logAndNotify(appName, req.path, error.message, additionalData, slackChannel);
    } else {
        /** If this is an expected error, log to papertrail, with any additional data */
        handlers.logAndNotify(appName, req.path, error.message, additionalData);
    }

    return next(error);
}

/** 
 * Middleware that handles errors for all routes in the captcha controller.
 * This middleware does not send back any error messages to the client
 * EXCEPT for when an HttpError is used, this is for increased security and not exposing
 * internal errors. 
 *
 * @async
 * @param {Error|import("./errorTypes").OperationalError|import('./errorTypes').HttpError} error - The error that occurred
 * @param {import("express").Request} req - The express request object
 * @param {import("express").Response} res - The express response object
 * @param {import("express".NextFunction)} next - The express next function
 * @returns {Promise<void>}
 */
async function expressHandleErrors(error, req, res, next) {
    /** 
     * If we have already sent a reply to the client 
     * let the express default error handling take over
     * @see {@link https://expressjs.com/en/guide/error-handling.html}
     * */
    if (res.headersSent) {
        return next(error);
    }

    /** 
     * If the error is an {@see errorTypes.HttpError} send the status code, and the message back
     * to the client. We don't send back the addition data in error.data though, that is only
     * meant for internal logging in {@see logErrors}
     * */
    if (error instanceof errorTypes.HttpError) {
        res.status(error.statusCode).send(error.message);
    } else {
        /** 
         * In dev mode, send back the stack trace
         * otherwise, send back a generic error message.
         * */
        if (process.env.NODE_ENV === "development") {
            return res.status(500).send(error.stack);
        } else {
            return res.status(500).send("Internal Server Error");
        }
    }
}

module.exports = {
    expressLogErrors,
    expressHandleErrors,
    _getNonOperationalSlackChannel,
    _isSlackChannelValid
};

==> fferrorhandler-2.0.4.tgz: package/middleware.js <==
const handlers = require("./handlers");
const errorTypes = require("./errorTypes");
const { slackChannels } = require("./constants");

const appName = process.env.PAPERTRAIL_PROGRAM || "UNKNOWN";

/**
 * Checks that a slack channel exist in {@link slackChannels}
 * @param {string} - The slack channel to check against {@link slackChannels}
 * @returns {boolean} - True for valid, false for invalid
 */
function _isSlackChannelValid(slackChannel) {
    return Object.values(slackChannels).includes(slackChannel);
}

/** 
 * Gets the slack channel to send the error to for {@link errorTypes.NonOperationalError}. 
 *
 * If the slack channel is set in the errorTypes.NonOperationalError constructor (error.slackChannel), it will use that. 
 * If it is not set on the error, it will check to see if process.env.NON_OPERATIONAL_SLACK_CHANNEL is set, and will use that if present.
 * If neither of those is set, it will default to alerts_warning. 
 *
 * It also validates that the slack channel passed in is a valid slack channel that exist in {@link slackChannels}
 * @param {errorTypes.NonOperationalError} error - The NonOperationalError to get the slack channel for 
 * @returns {string} - The slack channel to send the error to. 
 * @private
 * */
function _getNonOperationalSlackChannel(error) {
    const errorSlackChannel = error.slackChannel;
    const isSlackChannelSetOnError = (
        errorSlackChannel !== undefined &&
        errorSlackChannel !== null
    );

    if (isSlackChannelSetOnError) {
        const isSlackChannelValid = module.exports._isSlackChannelValid(errorSlackChannel);
        if (!isSlackChannelValid) {
            console.error(`Invalid slack channel set on error.slackChannel, defaulting to ${slackChannels.WARNING}`);
            return slackChannels.WARNING;
        } else {
            return errorSlackChannel;
        }
    }

    const envSlackChannel = process.env.NON_OPERATIONAL_SLACK_CHANNEL;
    const isSlackChannelSetInEnv = (
        envSlackChannel !== undefined &&
        envSlackChannel !== null
    );
    if (isSlackChannelSetInEnv) {
        const isSlackChannelValid = module.exports._isSlackChannelValid(envSlackChannel);
        if (!isSlackChannelValid) {
            console.error(`Invalid slack channel set in process.env.NON_OPERATIONAL_SLACK_CHANNEL, defaulting to ${slackChannels.WARNING}`);
            return slackChannels.WARNING;
        } else {
            return envSlackChannel;
        }
    }

    return slackChannels.WARNING;
}

/**
 * Middleware that handles logging errors to papertrail, and sending non operational errors to slack. 
 * @async
 * @param {Error|import("./errorTypes").OperationalError|import("./errorTypes").NonOperationalError|import("errorTypes").HttpError} error - The error that occurred
 * @param {import("express").Request} req  - The express request object
 * @param {import("express").Response} res  - The express response object
 * @param {import("express".NextFunction)} next - The express next function
 * @returns {Promise<void>}
 * */
async function expressLogErrors(error, req, res, next) {
    /** 
     * Custom error types in {@see errorTypes} combine the stack with optional additional data and put it on error.data 
     * if we have error.data, use that as the error details, if we don't, use the error stack by itself.
     * @type {string}
     * */
    const additionalData = error.data || error.stack;

    if (!error.isOperational) {
        const slackChannel = _getNonOperationalSlackChannel(error);
        /** If this is an unexpected error, set the slack channel and send to slack and papertrail */
        handlers.logAndNotify(appName, req.path, error.message, additionalData, slackChannel);
    } else {
        /** If this is an expected error, log to papertrail, with any additional data */
        handlers.logAndNotify(appName, req.path, error.message, additionalData);
    }

    return next(error);
}

/** 
 * Middleware that handles errors for all routes in the captcha controller.
 * This middleware does not send back any error messages to the client
 * EXCEPT for when an HttpError is used, this is for increased security and not exposing
 * internal errors. 
 *
 * @async
 * @param {Error|import("./errorTypes").OperationalError|import('./errorTypes').HttpError} error - The error that occurred
 * @param {import("express").Request} req - The express request object
 * @param {import("express").Response} res - The express response object
 * @param {import("express".NextFunction)} next - The express next function
 * @returns {Promise<void>}
 */
async function expressHandleErrors(error, req, res, next) {
    /** 
     * If we have already sent a reply to the client 
     * let the express default error handling take over
     * @see {@link https://expressjs.com/en/guide/error-handling.html}
     * */
    if (res.headersSent) {
        return next(error);
    }

    /** 
     * If the error is an {@see errorTypes.HttpError} send the status code, and the message back
     * to the client. We don't send back the addition data in error.data though, that is only
     * meant for internal logging in {@see logErrors}
     * */
    if (error instanceof errorTypes.HttpError) {
        res.status(error.statusCode).send(error.message);
    } else {
        /** 
         * In dev mode, send back the stack trace
         * otherwise, send back a generic error message.
         * */
        if (process.env.NODE_ENV === "development") {
            return res.status(500).send(error.stack);
        } else {
            return res.status(500).send("Internal Server Error");
        }
    }
}

module.exports = {
    expressLogErrors,
    expressHandleErrors,
    _getNonOperationalSlackChannel,
    _isSlackChannelValid
};
